;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xb429ad030ea09e1bb103111574627236 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x49ce87b2683f2d5069f52402217cb138 . "./code.cg2js/cg-printing.fasl")
(#x301e38c75b628f8855cde161f02ff2b7 . "./code.cg2js/cg-caret.fasl")
(#x589c5a094ce577375c69adbc2818843d . "./code.cg2js/cg-item-list.fasl")
(#x9559b72e67ff835907dd6ac34c482ee0 . "./code.cg2js/cg-progress-indicator.fasl")
(#x6f71bd504cb1955358f1f0c3440362e2 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x01640a432edc543a23c45880c28b320e . "./code.cg2js/cg-outline.fasl")
(#x6b79a21f400e13af92e68c6237b5c3f1 . "./code.cg2js/cg-palette.fasl")
(#xe303c85791dcc177bdf75fe7faba1afa . "./code.cg2js/cg-selected-object.fasl")
(#x96e3b7fca10c0cfe2dd8954516f41ffd . "./code.cg2js/cg-status-bar.fasl")
(#xfb9bfc5d630f5f537679683f87503c51 . "./code.cg2js/cg-wrap-string.fasl")
(#xeebb60dbbce43412fde166652c3a5660 . "./code.cg2js/cg-fill-texture.fasl")
(#x1d43b18c2ac8973bc069b01ce69ca31d . "./code.cg2js/cg-layout.fasl")
(#x973bb8c0b936d51f1a39eb2480bbf7a0 . "./code.cg2js/cg-timer.fasl")
(#x86d3b81adf2ce05192ec1c1cf817adc1 . "./code.cg2js/cg-dialog-item.fasl")
(#x0f136a994b765bde1c31fca903a3589e . "./code.cg2js/cg-button.fasl")
(#x5554ada52a2ed06fb299dbb03f26dec7 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x4e5faa1c64b5379885ba9af28fa67138 . "./code.cg2js/cg-tray.fasl")
(#x9e3f705d2d12cf4a46dcaaad4146ca2f . "./code.cg2js/cg-plot-widget.fasl")
(#x393b98c7231ea03d923f3b29d95eec5c . "./code.cg2js/cg-pixmap.fasl")
(#x70eaffa2bb1673221365b4d7e86003df . "./code.cg2js/cg-font-dialog.fasl")
(#x388012476018974ffb4a5f3f6b4ed6b4 . "./code.cg2js/cg-os-widget.fasl")
(#x6833d4d966e7151c40ac4e8897adcd07 . "./code.cg2js/cg-text-or-combo.fasl")
(#x515d7219ce43d6b9062993077f6a883c . "./code.cg2js/cg-trackbar.fasl")
(#x1aab686d2915f5b076bbfb7bb087d8da . "./code.cg2js/cg-up-down-control.fasl")
(#x294d0dc74c7d6612185d1b896947f5e9 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xe05f1382b20de959a7cb294661c6f6ee . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x63372daf185b7247b440422c61393109 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xdc3e45483db3576a812a3c395c07a6a1 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x3b0ffceab495d2048682a023316fde11 . "./code.cg2js/cg-chart-widget.fasl")
(#x0861f2cf5b6bc333a1c87e1644b29333 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x93f33ef63955e1aa25fa9d46292a8916 . "./code.cg2js/cg-group-box.fasl")
(#xe3ef91762f62d1c645d755582ca97b98 . "./code.cg2js/cg-static-text.fasl")
(#x4b2fa0ffd07e0beb41c90f664cd302b9 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x046f51302169fc7f5e859be1c0262616 . "./code.cg2js/cg-color-dialog.fasl")
(#xeb4413b800ec7aec7a4662749b2edcf2 . "./code.cg2js/cg-intersect.fasl")
(#xfba358fb8647158ef143d002a953df0a . "./code.cg2js/cg-gesture-emulation.fasl")
(#xe67e13af9d4dd0623a6cad4927062f2c . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x7a8b8830d93350040d0ff26948038ee7 . "./code.cg2js/cg-html-dialog.fasl")
(#xd30d9b8c4f6f68c1af3e75892b0c139e . "./code.cg2js/cg-lettered-menu.fasl")
(#xf6f034ecfaca8c375cde31446fcaef13 . "./code.cg2js/cg-class-grid.fasl")
(#x264bf1685e6bbf0ed7577b1242abf6da . "./code.cg2js/cg-pixmap-printing.fasl")
(#xea834f0a299b1ff02151def1e02c3eb2 . "./code.cg2js/cg-calendar.fasl")
(#xfbd19e3f1d6dade2f3de0cd7fe63058e . "./code.cg2js/cg-editable-text.fasl")
(#xc462261783f06da5a4e436a31367a14f . "./code.cg2js/cg-property.fasl")
(#x124bbe8f8b98190e8f7a324239e17a92 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x1cbf23914629f76126f17db61d8fd76e . "./code.cg2js/cg-grid-widget.fasl")
(#x4ae570980ddecd47b81165e1aa3dee8a . "./code.cg2js/cg-paren-matching.fasl")
(#xd6200ae9e9e8605105c8e71917fa1a73 . "./code.cg2js/cg-dev.fasl")
(#xf307c5f2402af833a6031639284d0277 . "./code.cg2js/cg-drawable.fasl")
(#xc1110656b526ce4d6fe4748e28a8275e . "./code.cg2js/cg-curve.fasl")
(#x6a42b76570b91792c8940b38fc7d28b7 . "./code.cg2js/cg-lisp-widget.fasl")
(#x3a40bc707a9dff538920dba9e668c9d6 . "./code.cg2js/cg-dde-utils.fasl")
(#x6f913dd9e0f73c599a1e15d0b8b38011 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x52f0902f99515cfb01f6d20755f9591b . "./code.cg2js/cg-object-editor-layout.fasl")
(#x4a35658e10e6bf5c25850cefa30fb7b5 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xc79a7e0bdaf0e3417a40b4634b6efd53 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xa788ae303f8e9cd733b9fe901213f121 . "./code.cg2js/cg-lamp.fasl")
(#xb53548f0d7de297a9b6fd633dfbc66cd . "./code.cg2js/cg-common-control.fasl")
(#x7b7c4ab551420493c1a3d6376975fb57 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x380cdaa096e8bd4d99f38500705f4b92 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xe60378b5cdbeb944ba61e42960c46212 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xf7ee8fe7c03589e056f6bc8b550096cc . "./code.cg2js/cg-project-window.fasl")
(#xaa442cf74411d556cddb9b3fb85847bd . "./code.cg2js/cg-sample-file-menu.fasl")
(#xbd8f097315d4a89e52f5f5845f01a989 . "./code.cg2js/cg-class-support.fasl")
(#x09bd5934e24069dedc5b44f8e2d848e2 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x0810525646b99f94482f6a4faa3128d6 . "./code.cg2js/cg-yes-no-list.fasl")
(#xa14a9fbe5aabd87eac64d25fbaf14ea6 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xf62ce2652faaf92216261f8d34f51453 . "./code.cg2js/cg-string-dialog.fasl")
(#x3e8941c081e0e975fa7b9087bc1dcfb0 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xb30fb0aa4068e6e38fdcd3234f53a616 . "./code.cg2js/cg-dropping-outline.fasl")
(#xa9dd12939a1769bde25e7a5751d4b95d . "./code.cg2js/cg.fasl")
(#xe49fb1bb5677eaa16eaf792d96f80aa6 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xa1dfb221986d89d7343fbcd489274bd4 . "./code.cg2js/cg-scaling-stream.fasl")
(#x15f5c794097f2d4c4ae852bc5f9dcca8 . "./code.cg2js/cg-yes-no-string.fasl")
(#x1143c2d698f2a3a8cbf859d91161a617 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xdf33480e185754528246417906733692 . "./code.cg2js/cg-web-browser.fasl")
(#x5c0f179b14da809e5c92bc4bf39444df . "./code.cg2js/cg-clipboard-stack.fasl")
(#xa48e94bcff4779a1436f7e26eb41f731 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x65d2a54b2ae3395cfe097f4ac8d8c35e . "./code.cg2js/cg-message-dialog.fasl")
(#xa88275d11aa3872431e13f08b87f51b7 . "./code.cg2js/cg-comtab.fasl")
(#x3bb6f265cb62c062df8a90773b1e1c09 . "./code.cg2js/cg-base.fasl")
(#x93926235a9bb723131c25c23f3c21661 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x08a5a5dca562be7f4550378ba7cbfc3c . "./code.cg2js/cg-text-edit-pane.fasl")
(#x743589bacad906d6c4d8d1c5612fb50f . "./code.cg2js/cg-utility-dialog.fasl")
(#xfafb835bcc61970a50e2745387fbe21f . "./code.cg2js/cg-object-editor.fasl")
(#xf57f0fa5c30acc5cacf10fc432d91afa . "./code.cg2js/cg-edit-in-place.fasl")
(#x8ac98ff5b24cfb81e1f89ca5fabe4c11 . "./code.cg2js/cg-radio-button.fasl")
(#x75a9c4845f57fa772b0499a584faa24d . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xf669f5b3abf435c9884229c7e1b66886 . "./code.cg2js/cg-nodes.fasl")
(#x81f6384e3ceccd8c27aa66f9bd8c37b6 . "./code.cg2js/cg-get-position.fasl")
(#x1869b044772009fe53abf3aa16b54b09 . "./code.cg2js/cg-directory-dialog.fasl")
(#xca3542e14f311e2c8b38a433db2afe2f . "./code.cg2js/cg-lisp-text.fasl")
(#x4ee7ca9a134ee3090f51157966be743d . "./code.cg2js/cg-combo-box.fasl")
(#x231e4aa0ee75adc683ed366431e3e726 . "./code.cg2js/cg-text-widget.fasl")
(#xb0f4d8f1936e01978bbe293b9aa803f4 . "./code.cg2js/cg-list-view.fasl")
(#x62d2fc9b730c75896164a6a4db3fc78e . "./code.cg2js/cg-graphics-context.fasl")
(#x81421b0d26d6173bcb9b5d09bff6e387 . "./code.cg2js/cg-get-pixmap.fasl")
(#x22a8e0f6cfd6443f0952d6a62bde8e01 . "./code.cg2js/cg-menu.fasl")
(#xa9788ea5871a64d0f4c925d6360157d1 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x89b011991d923c6e2473e5b24df5efa8 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x67f03109a2ad62722dfb983db6107d4f . "./code.cg2js/cg-tooltip.fasl")
(#x14716a92dededa5b0943bb322a2265f4 . "./code.cg2js/cg-html-widget.fasl")
(#x9ba70f9ee976ca2c09aacb06902667df . "./code.cg2js/cg-choice-list.fasl")
(#xb5c8fe16a7d06a0e60f17dfd0a3048d8 . "./code.cg2js/cg-icon.fasl")
(#x3fb8ce219e5dcfd394da834062c347c0 . "./code.cg2js/cg-acache.fasl")
(#xf3d74a399ad7e60f7ee26ac9e92166c7 . "./code.cg2js/cg-template-string.fasl")
(#x331af45d077a745caa20165f6a9a180c . "./code.cg2js/cg-bitmap-pane.fasl")
(#xd386502d7fdaae68756f04a05aec120f . "./code.cg2js/cg-toggling-widget.fasl")
(#xc1e34dfdb72ef6ad1b434219fabae877 . "./code.cg2js/cg-tab-control.fasl")
(#xabb59649fa103cd8b8340fa000b6182c . "./code.cg2js/cg-split-bar.fasl")
(#x2cae60a2673a307e8cff4cf9aebdec05 . "./code.cg2js/cg-picture-widget.fasl")
(#xb2ceb7e3f4128ec1f0d6b9c0dc61dd11 . "./code.cg2js/cg-player.fasl")
(#x55fe2a6bb3811083ac258d1bd6651e06 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xe56606766aa4f1d1636813410a3f7bf8 . "./code.cg2js/cg-choose-printer.fasl")
(#x5b32377cd1399f724eff432b7ebed927 . "./code.cg2js/cg-os-window.fasl")
(#x246535d0b854d48ae855feec588e6cf7 . "./code.cg2js/cg-scroll-bar.fasl")
(#x8e8725706d6732e73bb54e0782051005 . "./code.cg2js/cg-clipboard.fasl")
(#x429fc777c431fd77974447a97915ba34 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x1eeef531b8eb2b490a5165bc9c70f450 . "./code.cg2js/cg-hotspot.fasl")
(#xfa751719da399611fb17c6f184456500 . "./code.cg2js/cg-check-box.fasl")
(#x87fac6ac0e897d71fb7db6b3ca75240c . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x80e3cf282662eabcf2e171770b6ff1f0 . "./code.cg2js/cg-file-dialog.fasl")
(#xb4cad0974f066870272fb57bfb0280b5 . "./code.cg2js/cg-toolbar.fasl")
(#xb8a698231ee444033e320b5fab7919b3 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xdac5a9036534b768b5bb2d87cce9a673 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xb363b2461472c8696f7cbeeb5292e3cc . "./code.cg2js/cg-shortcut-menu.fasl")
(#x18f42e54c60148ceda22ef1a54a31b7c . "./code.idejs/ide-project.fasl")
(#xe3cc2a4a85d509c822272ec70ac07c3f . "./code.idejs/ide-intro.fasl")
(#xf062931fffc30630ff57c90acb931fa5 . "./code.idejs/ide-find-in-files.fasl")
(#xaa35aab12e7432f4011491cdb6bbba45 . "./code.idejs/ide-grapher.fasl")
(#x5e49c5ba8d8ae74cf7ac5cdcd6951bfe . "./code.idejs/ide-inspector.fasl")
(#xaa1aa0ff3eb7902036a02c6b6d7fe7bf . "./code.idejs/ide-editor.fasl")
(#x0f9ee92d60fde77e577f3360a01bef8b . "./code.idejs/ide-window-list.fasl")
(#x369882a7421922fa83c6a416330868fd . "./code.idejs/ide-process-browser.fasl")
(#xd4ec49d7c315e60ad1952705a6336d98 . "./code.idejs/ide-patch-dialog.fasl")
(#xc497c7f7b74ea66495fb64bc679c009c . "./code.idejs/ide-class-grapher.fasl")
(#xcedb70a24a0d53d5d259f3dbe0b6f5a6 . "./code.idejs/ide-base.fasl")
(#xeb85088204a109564fa3ed24c03fd89b . "./code.idejs/ide.fasl")
(#x5007d2fb9f4a60d188513111d1448a64 . "./code.idejs/ide-trace.fasl")
(#x99683dad406fec1cd7dada1058016d38 . "./code.idejs/ide-form.fasl")
(#xb946d2df8a23e26f82d464e2d4153a8e . "./code.idejs/ide-find-definitions.fasl")
(#x423c2d6dd237f137c59e287a96237f4a . "./code.idejs/ide-apropos.fasl")
(#x835011d53e449159e8e4a0de985ed4b2 . "./code.idejs/ide-class-browser.fasl")
(#x51688886417898322d03cf76e391e5fe . "./code.idejs/ide-navigator.fasl")
(#xb67ba1046966cf360a0706d626712143 . "./code.idejs/ide-clipboard.fasl")
(#xd6c9516d9f05b2500086d40bb5ab1a20 . "./code.idejs/ide-acache.fasl")
(#xfb6822ea29a59925a12a15287603f194 . "./code.idejs/ide-keyshort.fasl")
(#x8e92e1a3500906680b1d8483adc457a8 . "./code.idejs/ide-package-list.fasl")
(#x1b6102a74afe4c3221b89308af053a99 . "./code.idejs/ide-dev.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
